declare DEPLOY HELP BUILD MAIN_FILE OUTPUT_FILE HEX_FILE PROCESSOR_TYPE CLOCK_FREQUENCY AVR_DEVICE
declare -a files

#========= CHANGE THESE ACCORDING TO YOUR PARTICULAR PROCESSOR =============
PROCESSOR_TYPE=atmega644p   # used with the avr-gcc -mmcu parameter
CLOCK_FREQUENCY=12000000    # used with the avr-gcc -DF_CPU parameter
AVR_DEVICE=m644p            # used with the avrdude -p parameter
#===========================================================================

function usage() {
    echo ""
    echo "########### Build Script for AVR #########"
    echo
    echo "Author: Lenka Pitonakova - contact@lenkaspace.net"
    echo " "
    echo "Usage: ./build.sh fileName"
    echo "  Where fileName is name of your .c file, without the .c extension."
    echo "  The output file will be in format fileName.hex and will placed in the "$BUILD" directory."
    echo " "
    echo "Additionally, you can use these options: "
    echo "  -d : Deploy file to the device through USB using avrdude"
    echo " "
    echo "Examples:"
    echo "  ./build.sh helloWorld"
    echo "  ./build.sh -d helloWorld"
    echo
    echo "###########################################"
    echo ""
}

function compile() {
    echo ">> Compiling "$MAIN_FILE "..."
    if [ -d $BUILD ]; then
        rm -rf $BUILD
    fi
    mkdir $BUILD

    #-- compile
    avr-gcc -mmcu=$PROCESSOR_TYPE -DF_CPU=$CLOCK_FREQUENCY -Wall -Os $MAIN_FILE -o $BUILD/$OUTPUT_FILE
    #-- change to hex file for avr and delete the output file
    avr-objcopy -O ihex $BUILD/$OUTPUT_FILE $BUILD/$HEX_FILE
    rm $BUILD/$OUTPUT_FILE
}

function deploy() {
    echo ">> Deploying " $MAIN_FILE "..."
    avrdude -c usbasp -p $AVR_DEVICE -U flash:w:$BUILD/$HEX_FILE
}


DEPLOY=0
HELP=0
BUILD="build"

#-- check if help should be displayed
while getopts "hd" name
do
    case $name in
    h)
    HELP=1;;
esac
done

if [ $HELP -eq 1 ]; then
    usage;
    exit 1;
fi

#-- check all the other parameters
eval set -- $(getopt -n $0 -o "-d:" -- "$@")
while [ $# -gt 0 ] ; do
    case "$1" in
        -d) DEPLOY=1 ; shift ;;
        --) shift ;;
        -*) echo "bad option '$1'" ; exit 1 ;;
        *) files=("${files[@]}" "$1") ; shift ;;
    esac
done


#-- check if file supplied
if [ ${#files} -eq 0 ] ; then
    echo "Output file required. Use option -h for help."
    exit 1
fi

#-- set the files
MAIN_FILE="${files[@]}.c"
OUTPUT_FILE="${files[@]}.elf"
HEX_FILE="${files[@]}.hex"

#-- compile and deploy
compile

if [ $DEPLOY -eq 1 ]; then
    deploy
fi




